/*
  CCL -- Configurable Custom Logic Example. LUT1 is used, which isn't even
  connected to the Arduino Nano Every board pins! Output (PC3) will be high
  in the state PC2 PC1' PC0'. The pins PC2 through PC0 are driven in the loop function
  through all 8 combinations. An interrupt will occur when the output goes high.
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */
void setup() {
  Serial.begin(9600);
  delay(800);
  VPORTC.DIR |= 0x0f; // Make them all output pins
  CCL.LUT1CTRLB = CCL_INSEL0_IO_gc | CCL_INSEL1_IO_gc;
  CCL.LUT1CTRLC = CCL_INSEL2_IO_gc;
  CCL.TRUTH1 = 0x10; // Only high output for input 1 0 0
  CCL.LUT1CTRLA = CCL_OUTEN_bm | CCL_ENABLE_bm;
  CCL.INTCTRL0 = 1 << 2; // Rising edge interrupt
  CCL.CTRLA = 1; // Enable the CCL
}

void loop() {
  static uint8_t counter;
  VPORTC.OUT = (VPORTC.OUT & 0xf0) | counter;
  Serial.print("In: ");
  Serial.print(counter, BIN);
  Serial.print(" Out: ");
  Serial.println((VPORTC.IN & 0x08) != 0);
  counter = (counter + 1) & 0x7; // count modulo 8
  if (counter == 0) delay(1000);
}

ISR(CCL_CCL_vect) {
  Serial.println("Triggered!");
  CCL.INTFLAGS = 0x2;
}
