/*
  Nano_Every_Progmem -- Demonstration of PROGMEM use and non-use in ATmega4809
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */

// All const variables are in program memory in the ATmega4809
const PROGMEM  char hstring[] = "Hello World!";
const PROGMEM int16_t mytable[] = {10, 20, 30, 50, 100};

// These won't be in program memory with an ATmega328P, but will be
// with an ATmega4809 because of "const"
const  char hstringNew[] = "Hello World!";
const  int16_t mytableNew[] = {10, 20, 30, 50, 100};
const char * const giant[] = {"fee ", "fie ", "foe ", "fum\n"};

void setup() {
  char buffer[13];
  Serial.begin(9600);
  delay(700);

  // Using PROGMEM
  Serial.println(F("Hello World!"));
  Serial.println(strcpy_P(buffer, PSTR("Hello World!")));
  Serial.println(strcpy_P(buffer, hstring));
  Serial.println(pgm_read_word(&mytable[1]));

  // Not Using PROGMEM
  Serial.println("Hello World!");
  Serial.println(hstringNew);
  Serial.println(mytableNew[1]);
  for (int i = 0; i < 4; i++) Serial.print(giant[i]);

  // See where they are stored.
  Serial.println((unsigned)&"Hello World", HEX);
  Serial.println((unsigned)hstring, HEX);
  Serial.println((unsigned)mytable, HEX);
  Serial.println((unsigned)hstringNew, HEX);
  Serial.println((unsigned)mytableNew, HEX);
  Serial.println((unsigned)giant, HEX);  // The array of strings
}

void loop() {
  // put your main code here, to run repeatedly:

}
