/* RTC -- Real-Time Counter and Periodic Interrupt Timer Demonstration
    turn the LED on or off every second. Send "tick" and "tock" to the terminal
    every second, spaced apart 1/2 second.
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */
#include <Pins.h>
void setup() {
  ddr.digital_13 = 1; // LED pin
  Serial.begin(9600);
  RTC.CLKSEL = 0; // Internal 32.768kHz clock
  RTC.PER = 32767; // Gives a period of 32768
  RTC.CMP = 16384; // Compare half way through
  RTC.INTCTRL = 3; // Enable both CMP and OVF interrupts
  RTC.CTRLA = 1; // Enable RTC, prescaler is off
  RTC.PITINTCTRL = 1; // Enable PI interrupt
  RTC.PITCTRLA = RTC_PERIOD_CYC16384_gc + 1; // This will be half a second. Enable PIT
}

void loop() {

}

ISR(RTC_CNT_vect) {
  if ((RTC_INTFLAGS & 1) != 0) { // OVF
    RTC.INTFLAGS = 1; // clear interrupt
    Serial.println("...Tock");
  } else if ((RTC.INTFLAGS & 2) != 0) { // CMP
    RTC.INTFLAGS = 2; // clear interrupt
    Serial.print("Tick...");
  }
}

ISR(RTC_PIT_vect) {
  pin.digital_13 = 1; // Toggle LED
  RTC.PITINTFLAGS = 1; // clear interrupt
}
