/*
  SPI_MASTER -- Three byte transfer SPI Master example. Connect
  to external shift register or series of shift registers as
  shown in the book.
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */

#include <Pins.h>
static uint8_t dataOut[3] = {0x55, 0xaa, 0}; // data to send
static uint8_t dataIn[3] = {0, 0, 0};
volatile uint8_t dataIndex = 0;
void setup() {
  Serial.begin(9600);
  PORTMUX.TWISPIROUTEA |= PORTMUX_SPI0_ALT2_gc;
  ddr.digital_8 = 1; // SS pin
  port.digital_8 = 1; // Normally high
  ddr.digital_11 = 1; // MOSI pin
  //ddr.digital_12 = 0; // MISO pin
  ddr.digital_13 = 1; // SCK
  SPI0.CTRLA = 0x23; // MASTER, /16, ENABLE
  SPI0.CTRLB = 0; // Buffering off, CPOL=0, CPHA=0
  SPI0.INTCTRL = 1; // Enable interrupt
}

void loop() {
  dataOut[2]++; // Change value to store externally every cycle
  dataIndex = 0;
  port.digital_8 = 0; // Enable SS
  SPI0.DATA = dataOut[0]; // transmit first byte.
  delay(500);
  Serial.print("Received:");
  for (uint8_t i = 0; i < 3; i++) {
    Serial.print(" ");
    Serial.print(dataIn[i], HEX);
  }
  Serial.println("");
}

ISR(SPI0_INT_vect) { // receive and transmit bytes
  dataIn[dataIndex] = SPI0.DATA; // receive next byte
  dataIndex++;
  if (dataIndex < 3) { // more to send
    SPI0.DATA = dataOut[dataIndex];
  }  else {
    port.digital_8 = 1; // Disable SS
  }
  SPI0.INTFLAGS = 0x80; // Reset interrupt flag
}
