/* Timer/Counter A FRQ Mode. Generates a square wave of a configurable frequency
    We can get output from overflow and CMP0 interrupts or the WO0 pin.
    Using 16MHz clock for comparason with Uno/Nano
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */
#include <Pins.h>
void setup() {
  ddr.digital_9 = 1; // WO0
  ddr.digital_10 = 1; // CMP0 toggles this
  ddr.digital_2 = 1; // OVF toggles this
  TCA0.SINGLE.CMP0BUF = 24; // Sets square wave period to (24 + 1)*2
  // Timer/Counter clocks, 5 kHz
  TCA0.SINGLE.INTCTRL = 0x11; // OVF and CMP0 interrupts
  TCA0.SINGLE.CTRLB = 0x11; // Enable CMP0 and FRQ mode
}

void loop() {
  // put your main code here, to run repeatedly:
}

ISR(TCA0_OVF_vect) {
  pin.digital_2 = 1;
  TCA0.SINGLE.INTFLAGS = 1; // reset flag
}

ISR(TCA0_CMP0_vect) {
  pin.digital_10 = 1;
  TCA0.SINGLE.INTFLAGS = 0x10; // reset flag
}
