/* TCA_PW -- Capture Pulse Width using Timer/Counter A
    IMPORTANT -- this requires the "BETTER OPERATION" modifications that
    run the System Time Tracking off of the system clock rather than the
    TCA prescaler.
    Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */

#include <Pins.h>
void setup() {
  Serial.begin(9600);
  delay(800);
  analogWrite(3, 64); // 8 us high 24 us low
  TCB1.CTRLA = 3; // 0.125 us rate
  EVSYS.CHANNEL4 = EVSYS_GENERATOR_PORT1_PIN5_gc; // Route pin 3, PF5
  EVSYS.USERTCA0 = EVSYS_CHANNEL_CHANNEL4_gc; // to TCA0
  TCA0.SINGLE.CTRLA = TCA_SINGLE_CLKSEL_DIV1_gc | 1; // Run TCA at 16MHz.
  TCA0.SINGLE.PERBUF = 0xffff; // Allow maximum count
  TCA0.SINGLE.CTRLB = TCA_SINGLE_WGMODE_NORMAL_gc; // Normal mode.
  TCA0.SINGLE.EVCTRL = TCA_SINGLE_EVACT_HIGHLVL_gc | 1; // Count while high
  PORTF.PIN5CTRL = PORT_ISC_FALLING_gc; // Interrupt on pin 3 falling.
  Serial.println("Pulse width in clock ticks, should be 128");
}

volatile uint16_t width;
volatile uint16_t count;

void loop() {
  uint16_t tempVal;
  delay(500);
  cli();
  tempVal = width;
  sei();
  // Calculate time in system clock ticks and print
  Serial.println(tempVal);
}

ISR(PORTF_PORT_vect) {
  width = TCA0.SINGLE.CNT; // Get time high in counter ticks
  TCA0.SINGLE.CNT = 0; // And reset count
  intflg.digital_3 = 1; // clear interrupt flag
}
