/* TCB Capture on Event Mode
    Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */

#include <Pins.h>
void setup() {
  Serial.begin(9600);
  tone(3, 1000); // 1kHz tone on pin 3
  EVSYS.CHANNEL4 = EVSYS_GENERATOR_PORT1_PIN5_gc; // Route pin 3, PF5
  EVSYS.USERTCB0 = EVSYS_CHANNEL_CHANNEL4_gc; // to TCB0
  TCB0.CTRLA = 0; // Turn off channel for configuring
  TCB0.CTRLB = 0x2; // capture on event mode
  TCB0.EVCTRL = 0x41; // Enable input capture, rising edge of signal
  TCB0.INTCTRL = 1; // Enable interrupt
  TCB0.CTRLA = 1; // Enable and use system clock
}

volatile uint16_t period;

void loop() {
  uint16_t tempVal;
  delay(1000);
  cli();
  tempVal = period;
  sei();
  Serial.println((tempVal + 8) / 16);
}

ISR(TCB0_INT_vect) {
  static uint16_t lastTime;
  uint16_t thisTime = TCB0.CCMP; // reading CCMP clears interrupt flag
  period = thisTime - lastTime;
  lastTime = thisTime;
}
