/* TWI_MASTER_SLAVE -- This program demonstrates operation as both master and slave
    simultaneously. It combines master_reader, master_writer, slave_sender, and
    slave_receiver functionality in a single program. No external wiring is necessary although
    the SCL and SDA signals can be observed on A5 and A4 respectively.

  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */
extern "C" {
#include "twi.h"
}

#define SLAVE_ADDR 8

void receiveEvent(int);
uint8_t requestEvent(void);
uint8_t slaveReadBuf[8];
uint8_t slaveWriteBuf[] = "Hello ";

void setup()
{
  Serial.begin(9600); // Something requires setting up Serial after the Setup
  delay(800);
  TWI_MasterInit(100000L);
  TWI_SlaveInit(SLAVE_ADDR); // slave address SLAVE_ADDR
  TWI_attachSlaveRxEvent(receiveEvent, slaveReadBuf, sizeof(slaveReadBuf));
  TWI_attachSlaveTxEvent(requestEvent, slaveWriteBuf);
}

uint8_t data[6] = "x is "; // 5 characters, data[5] is counter
uint8_t dataIn[8];
void loop()
{
  TWI_MasterWrite(SLAVE_ADDR, data, 6, true); // Send 6 bytes to slave address SLAVE_ADDR
  data[5]++; // Next write will be different.
  TWI_MasterRead(SLAVE_ADDR, dataIn, 6, true);    // request 6 bytes from slave device SLAVE_ADDR
  Serial.print((char *)dataIn);
  delay(500);
}

// function that executes whenever data is received from master
// this function is registered as an event, see setup()
void receiveEvent(int howMany) {
  int charCount = howMany - 1;
  uint8_t *buf = slaveReadBuf;
  while (charCount-- > 0) Serial.print((char)*buf++);
  Serial.println(*buf);
}

// function that executes whenever data is requested by master
// this function is registered as an event, see setup()
uint8_t requestEvent() {
  return 6; // respond with message of 6 bytes
            // as expected by master
}
