/* WDT -- Show functioning of Watchdog Timer. Demonstrate windowing by
    defining RESETTOOSOON. 2.25 second total period after which reset occurs,
    however if RESETTOOSOON is defined wdt_reset is called in 200ms which
    causes a reset.
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */


#include <avr/wdt.h>
//#define RESETTOOSOON
void setup() {
  Serial.begin(9600);
  delay(800);

  CPU_CCP = CCP_IOREG_gc; // allow changes
  WDT.CTRLA = WDT_WINDOW_256CLK_gc | WDT_PERIOD_2KCLK_gc; // 256ms window 2 second period
  CPU_CCP = CCP_IOREG_gc; // allow changes
  WDT.STATUS = WDT_LOCK_bm; // lock it
  while (WDT.STATUS & WDT_SYNCBUSY_bm != 0) {} ; // wait for synchronizing complete
  wdt_reset(); // first reset enables windowing
}

void loop() {
#ifdef RESETTOOSOON
  Serial.println("reset too soon");
  delay(200); // delay is too short!
#else
  delay(300);
#endif
  wdt_reset(); // Program will run for an additional 1.25s from this point.

  Serial.println("");
  Serial.print("This will print ");
  while (1) {
    delay (200);
    Serial.print("*");
  }
  Serial.print("This certainly will not print");
}
