/*
  adc_1 - Measure the voltage on pin A2. Also get the internal
  temperature. Functions like analogRead()
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */
#include <Pins.h>
int getTemperature(void) {
  VREF.CTRLA = VREF_ADC0REFSEL_1V1_gc; // 1.1 volt reference
  ADC0.CTRLC = ADC_SAMPCAP_bm + 3; // VREF reference, correct clock divisor
  ADC0.MUXPOS = ADC_MUXPOS_TEMPSENSE_gc; // Select temperature sensor
  ADC0.CTRLD = 2 * (1 << ADC_INITDLY_gp); // Initial delay of 32us
  ADC0.SAMPCTRL = 31; // Maximum length sample time (32us is desired)
  ADC0.COMMAND = 1; // Start the conversion
  while ((ADC0.INTFLAGS & ADC_RESRDY_bm) == 0) ; // wait for completion
  // The following code is based on the ATmega4809 data sheet
  int8_t sigrow_offset = SIGROW.TEMPSENSE1; // Read signed value from signature row
  uint8_t sigrow_gain = SIGROW.TEMPSENSE0; // Read unsigned value from signature row
  uint16_t adc_reading = ADC0.RES; // ADC conversion result with 1.1 V internal reference
  uint32_t temp = adc_reading - sigrow_offset;
  temp *= sigrow_gain; // Result might overflow 16 bit variable (10bit+8bit)
  temp += 0x80; // Add 1/2 to get correct rounding on division below
  temp >>= 8; // Divide result to get Kelvin
  uint16_t temperature_in_K = temp;
  return temperature_in_K - 273; // Return Celsius temperature
}

int getA2(void) {
  ADC0.CTRLC = ADC_SAMPCAP_bm + 1 * (1 << ADC_REFSEL_gp) + 3; // VDD reference, correct clock divisor (16)
  ADC0.MUXPOS = ADC_MUXPOS_AIN1_gc; // This is input from pin A2
  ADC0.CTRLD = 0; // Turn off all delays (used in the temperature measurements)
  ADC0.SAMPCTRL = 0;
  ADC0.COMMAND = 1; // Start the conversion
  while ((ADC0.INTFLAGS & ADC_RESRDY_bm) == 0) ; // wait for completion
  return (ADC0.RES * 5000L) / 1024; // Return voltage in millivolts
}

void setup() {
  Serial.begin(9600);
  PORTD.PIN1CTRL = PORT_ISC_INPUT_DISABLE_gc; // Disable digital on A2 (Pin PD1)
  ADC0.CTRLA = 1; // Enable the ADC
  delay(700);
}

void loop() {
  Serial.print("Temperature is: ");
  Serial.println(getTemperature());
  Serial.print("Voltage is: ");
  Serial.println(getA2());
  delay(10000); // Wait 10 seconds per reading
}
