/*
  adc_3 -- Continously read A0 through A5 and report values every
  second. Uses an interrupt routine to direct the multiplexer
  and start operation.
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */


// Table of pins to read, expressed as values for ADC0.MUXPOS
#define NUMBER_INPUTS (6)
const uint8_t muxValues[NUMBER_INPUTS] = {ADC_MUXPOS_AIN3_gc,
                                          ADC_MUXPOS_AIN2_gc,
                                          ADC_MUXPOS_AIN1_gc,
                                          ADC_MUXPOS_AIN0_gc,
                                          ADC_MUXPOS_AIN12_gc,
                                          ADC_MUXPOS_AIN13_gc
                                         };

volatile uint16_t adcvalues[NUMBER_INPUTS]; // Room for reading all desired ADC values.
uint8_t muxIndex = 0; // Index of next value to read

void setup() {
  Serial.begin(9600);
  // Turn of digital input for all used analog pins
  PORTD.PIN3CTRL = PORT_ISC_INPUT_DISABLE_gc; // A0
  PORTD.PIN2CTRL = PORT_ISC_INPUT_DISABLE_gc; // A1
  PORTD.PIN1CTRL = PORT_ISC_INPUT_DISABLE_gc; // A2
  PORTD.PIN0CTRL = PORT_ISC_INPUT_DISABLE_gc; // A3
  PORTF.PIN4CTRL = PORT_ISC_INPUT_DISABLE_gc; // A4
  PORTF.PIN5CTRL = PORT_ISC_INPUT_DISABLE_gc; // A5
  PORTA.PIN4CTRL = PORT_ISC_INPUT_DISABLE_gc; // A4 tied to this pin
  PORTA.PIN5CTRL = PORT_ISC_INPUT_DISABLE_gc; // A5 tied to this pin
  ADC0.CTRLA = 1; // Enable ADC
  ADC0.CTRLC = ADC_SAMPCAP_bm + 1 * (1 << ADC_REFSEL_gp) + 3;
  // VDD reference, correct clock divisor (16)
  ADC0.MUXPOS = muxValues[muxIndex]; // This is input from pin A0
  ADC0.INTCTRL = 1; // Enable RESRDY interrupt
  ADC0.COMMAND = 1; // Start the conversion
  delay(700);
}


void loop() {
  int value;
  uint8_t index;
  for (index = 0; index < NUMBER_INPUTS; index++) {
    cli(); // Atomic access
    value = adcvalues[index];
    sei();
    Serial.print(value);
    Serial.print(" ");
  }
  Serial.println("");
  delay(1000);
}

ISR(ADC0_RESRDY_vect) { // A conversion has completed. Get result and start next conversion
  adcvalues[muxIndex] = ADC0.RES;
  muxIndex++;
  if (muxIndex >= NUMBER_INPUTS) {
    muxIndex = 0;
  }
  ADC0.MUXPOS = muxValues[muxIndex]; // This is input from next pin
  ADC0.COMMAND = 1; // Start the conversion
}
