/*
  comparator_test -- Analog Comparator Demonstration
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */

#include <Pins.h>

// This program uses the PWM on pin 3, which is filtered by
// a serial 10K resistor to pin A1, the positive input to the
// comparator, and a 2.2uF capacitor from there to ground.
// The negative input to the comparator is the internal DAC,
// set at 2.5 volts. The comparator interrupt triggers on both the rising and
// falling edges of the comparator outputPin 2 goes high when the comparator triggers
// and is reset when the ramp switches direction.
// The comparator output is also routed using the Event System to pin 5.

void setup() {
  // put your setup code here, to run once:
  analogWrite(3, 0);
  ddr.digital_2 = 1;
  PORTD.PIN2CTRL = PORT_ISC_INPUT_DISABLE_gc; // Disable digital on A1
  EVSYS.CHANNEL0 = EVSYS_GENERATOR_AC0_OUT_gc; // Connect AC to pin 5
  EVSYS.USEREVOUTB = EVSYS_CHANNEL_CHANNEL0_gc;
  VREF.CTRLA = VREF_AC0REFSEL_AVDD_gc; // Set voltage reference for AC DAC to VDD (5 volts)
  AC0.MUXCTRLA = 3; // A1 is positive, DAC is negative
  AC0.DACREF = 128; // Mid scale, 2.5 volts
  AC0.CTRLA = 0x0f; // Enable comparator with 50mv hysteresis, interrupt on both edges, low power
  AC0.INTCTRL = 1; // Enable AC interrupt
}

void loop() { // The trapezoidal waveform generator
  static uint8_t width = 0;
  static bool goingUp = true;
  analogWrite(3, width);
  if (goingUp) {
    if (width == 255) {
      goingUp = false;
      port.digital_2 = 0;
    }
  } else {
    if (width == 0) {
      goingUp = true;
      port.digital_2 = 0;
    }
  }
  width += (goingUp ? 1 : -1);
  delay(2);
}

ISR(AC0_AC_vect) {
  pin.digital_2 = 1; // toggle on match -- should always be
  // 0 -> 1 if sufficient hysteresis
  AC0.STATUS = 1; // Reset interrupt flag
}
