/*
  eeprom_test -- A simple test of reading and writing EEPROM. A structure of
  the user's eeprom memory layout is located at the EEPROM address.
  This program is intended for the ATmega4809 (Arduino Nano Every)
  Copyright (C) 2020  Thomas Almy

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

  Contact the author at tom@almy.us */

#include <avr/eeprom.h>
struct MYEEPROM {
  byte aByte;
  word aWord;
  word aSecondWord;
} * const foo = (struct MYEEPROM *) EEPROM_START;
// Write location is relative to start of EEPROM
// Read location is mapped into address space
#define eeprom_w  (* (struct MYEEPROM *) 0)
#define eeprom   (* (struct MYEEPROM *) EEPROM_START)

struct MYUSERROW {
  byte aByte;
  word aWord;
  word anArray[10];
};
// User Row is -256 bytes before the start of EEPROM
#define userrow_w (* (struct MYUSERROW *)0xFF00)
#define userrow (* (struct MYUSERROW *) USER_SIGNATURES_START)

void setup() {
  int i;
  // put your setup code here, to run once:
  Serial.begin(9600);
  delay(700);
  Serial.println("Before writing, contents:");
  Serial.println(eeprom.aWord);
  Serial.println(userrow.aWord);
  eeprom_write_word(&eeprom_w.aWord, 1234);
  eeprom_write_word(&userrow_w.aWord, 4321);
  for (i=0; i < 10; i++) {
    eeprom_write_word(&userrow_w.anArray[i], (i+1)*101);
  }

  Serial.println("After writing, contents:");
  Serial.println(eeprom.aWord);
  Serial.println(foo->aWord); // does the same thing as preceding line
  Serial.println(userrow.aWord);
  for (i=0; i < 10; i++) {
    Serial.print(userrow.anArray[i]);
    Serial.print(" ");
  }
  Serial.println("");
}

void loop() {
  // put your main code here, to run repeatedly:
}
